import sys
from tabulate import tabulate


class StdinConsole:
    def __init__(self, config_context):
        pass

    def run(self):
        while True:
            command = sys.stdin.read()
            if not command:
                break
            command = command.strip()  # Remove stdin enf of lines
            short_cmd = command.split()[0]
            result = self.cmd_handler.process(command)
            if result is None:
                print(f"Error: Command '{short_cmd}' is not defined", file=sys.stderr)

    def table(self, table_data):
        headers = table_data[0]
        values = table_data[1:]
        print(tabulate(values, headers=headers))



from collections import OrderedDict
from ..mapper import table_from_list


class Command_Nodes:
    callers = [".nodes", ".n"]
    description = "List the cluster nodes"

    default_fields = OrderedDict(
        {
            "Name": "metadata.name",
            "InternalIP": "status.addresses.[type=InternalIP].address",
            "Hostname": "status.addresses.[type=Hostname].address",
            "Ready": "status.conditions.[reason=KubeletReady].status",
            "Version": "status.node_info.kubelet_version",
        }
    )

    def run(self, console, api):
        response = api.list_node()
        response_data = table_from_list(response, self.default_fields)
        console.table(response_data)
        return "nodes"



from . import commands
import pkgutil
import importlib

# https://packaging.python.org/guides/creating-and-discovering-plugins/


def iter_namespace(ns_pkg):
    return pkgutil.iter_modules(ns_pkg.__path__, ns_pkg.__name__ + ".")


def load_commands():
    return {
        name: importlib.import_module(name)
        for finder, name, ispkg in iter_namespace(commands)
    }



""" Command handler """
import inspect


class CommandHandler:
    def __init__(self, commands, console, api):
        self.commands = commands
        self.console = console
        self.api = api
        self.callers = {}
        for module_name, module_obj in commands.items():
            cls_members = inspect.getmembers(module_obj, inspect.isclass)
            cls_members = [
                c[1] for c in cls_members if c[1].__name__.startswith("Command")
            ]
            for cmd in cls_members:
                cmd_obj = cmd()
                for caller in cmd.callers:
                    assert caller not in self.callers
                    self.callers[caller] = cmd_obj
        # Ugly but we need so that console can call us
        console.cmd_handler = self

    def process(self, command):
        handler = self.callers.get(command)
        if handler:
            return handler.run(self.console, self.api)



from prompt_toolkit import PromptSession
from prompt_toolkit.lexers import PygmentsLexer
from pygments.lexers.data import YamlLexer
from prompt_toolkit import print_formatted_text, HTML
from tabulate import tabulate
from prompt_toolkit.styles import Style


class TerminalConsole:
    def __init__(self, config_context):
        # Feed the autocompleter
        self.session = PromptSession()
        self.config_context = config_context
        self.toolbar_style = Style.from_dict(
            {
                "bottom-toolbar": "#aaaa00 bg:#ff0000",
                "bottom-toolbar.text": "#6A5ACD bg:#ffffff",
            }
        )

    def run(self):
        while True:
            command = self.session.prompt(
                ">> ",
                bottom_toolbar=self.get_toolbar,
                style=self.toolbar_style,
                refresh_interval=5,
                lexer=PygmentsLexer(YamlLexer),
            )
            if not command:  # Just skip empty enters
                continue
            short_cmd = command.split()[0]
            result = self.cmd_handler.process(command)
            if result is None:
                print_formatted_text(
                    HTML(
                        f"Error: <ansired>Command '{short_cmd}' is not defined</ansired>"
                    )
                )

    def get_toolbar(self):
        cluster = self.config_context["context"]["cluster"]
        user = self.config_context["context"]["user"]
        status = f" <b>{user}@{cluster}</b> "
        return HTML(status)

    def table(self, table_data):
        headers = table_data[0]
        values = table_data[1:]
        print_formatted_text(tabulate(values, headers=headers))



import sys
import click
from . import __version__
from .plugins import load_commands
from .consoles import TerminalConsole, StdinConsole
from .cmd_handler import CommandHandler
from .api import get_api, config


@click.command()
@click.version_option(version=__version__)
def main():
    try:
        api = get_api()
    except TypeError:
        print("ERROR: Unable to find KUBECONFIG file !!!")
        exit(2)
    commands = load_commands()
    console_class = TerminalConsole if sys.stdin.isatty() else StdinConsole
    console = console_class(config.list_kube_config_contexts()[1])
    CommandHandler(commands, console, api)
    console.run()



def find_item(root_item, field_spec):
    tokens = field_spec.split(".")
    cursor = root_item
    for tokenName in tokens:
        if tokenName[0] == "[":
            assert isinstance(cursor, list)
            tokenName = tokenName.strip("[]]")
            search_key, search_value = tokenName.split("=")
            match_item = [x for x in cursor if getattr(x, search_key) == search_value]
            if match_item:
                cursor = match_item[0]
            else:
                return None
        else:
            cursor = getattr(cursor, tokenName)
    return cursor


def table_from_list(data, fields):
    input_data = data.items
    response_data = []
    response_data.append(fields.keys())
    for list_item in input_data:
        row = []
        for field_mame, field_spec in fields.items():
            value = find_item(list_item, field_spec)
            row.append(value)
        response_data.append(row)
    return response_data



from setuptools import setup
import sys
import os
import io

assert sys.version_info >= (3, 6, 0), "requires Python 3.6+"
from pathlib import Path  # noqa E402

CURRENT_DIR = Path(__file__).parent


def setup_package():

    # Get readme
    readme_path = os.path.join(CURRENT_DIR, "README.md")
    with io.open(readme_path, encoding="utf8") as f:
        readme = f.read()

    setup(long_description=readme, long_description_content_type="text/markdown")


if __name__ == "__main__":
    setup_package()
